@@ -39,6 +39,8 @@ static GSList *g_driver_list = NULL;
 static GSList *g_modem_list = NULL;
 
 static int next_modem_id = 0;
+static gboolean powering_down = FALSE;
+static int modems_remaining = 0;
 
 enum ofono_property_type {
 	OFONO_PROPERTY_TYPE_INVALID = 0,
@@ -96,7 +98,7 @@ struct ofono_property {
 	void *value;
 };
 
-unsigned int __ofono_modem_alloc_callid(struct ofono_modem *modem)
+unsigned int __ofono_modem_callid_next(struct ofono_modem *modem)
 {
 	unsigned int i;
 
@@ -104,14 +106,18 @@ unsigned int __ofono_modem_alloc_callid(struct ofono_modem *modem)
 		if (modem->call_ids & (0x1 << i))
 			continue;
 
-		modem->call_ids |= (0x1 << i);
 		return i;
 	}
 
 	return 0;
 }
 
-void __ofono_modem_release_callid(struct ofono_modem *modem, int id)
+void __ofono_modem_callid_hold(struct ofono_modem *modem, int id)
+{
+	modem->call_ids |= (0x1 << id);
+}
+
+void __ofono_modem_callid_release(struct ofono_modem *modem, int id)
 {
 	modem->call_ids &= ~(0x1 << id);
 }
@@ -399,14 +405,20 @@ static int set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 	const struct ofono_modem_driver *driver = modem->driver;
 	int err = -EINVAL;
 
-	if (driver == NULL)
-		return -EINVAL;
-
 	if (modem->powered_pending == powered)
 		return -EALREADY;
 
+	/* Remove the atoms even if the driver is no longer available */
+	if (powered == FALSE) {
+		remove_all_atoms(modem);
+		modem->call_ids = 0;
+	}
+
 	modem->powered_pending = powered;
 
+	if (driver == NULL)
+		return -EINVAL;
+
 	if (powered == TRUE) {
 		if (driver->enable)
 			err = driver->enable(modem);
@@ -430,7 +442,18 @@ static gboolean set_powered_timeout(gpointer user)
 	DBG("modem: %p", modem);
 
 	modem->timeout = 0;
-	modem->powered_pending = modem->powered;
+
+	if (modem->powered_pending == FALSE) {
+		DBusConnection *conn = ofono_dbus_get_connection();
+		dbus_bool_t powered = FALSE;
+
+		modem->powered = FALSE;
+		ofono_dbus_signal_property_changed(conn, modem->path,
+						OFONO_MODEM_INTERFACE,
+						"Powered", DBUS_TYPE_BOOLEAN,
+						&powered);
+	} else
+		modem->powered_pending = modem->powered;
 
 	if (modem->pending != NULL) {
 		DBusMessage *reply;
@@ -461,6 +484,9 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT)
 		return __ofono_error_invalid_args(msg);
 
+	if (powering_down == TRUE)
+		return __ofono_error_failed(msg);
+
 	dbus_message_iter_recurse(&iter, &var);
 
 	if (g_str_equal(name, "Powered") == TRUE) {
@@ -499,8 +525,6 @@ static DBusMessage *modem_set_property(DBusConnection *conn,
 		if (powered) {
 			if (modem->driver->pre_sim)
 				modem->driver->pre_sim(modem);
-		} else {
-			remove_all_atoms(modem);
 		}
 
 		return NULL;
@@ -524,7 +548,6 @@ static GDBusSignalTable modem_signals[] = {
 void ofono_modem_set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
-	dbus_bool_t dbus_powered;
 
 	if (modem->timeout > 0) {
 		g_source_remove(modem->timeout);
@@ -544,25 +567,29 @@ void ofono_modem_set_powered(struct ofono_modem *modem, ofono_bool_t powered)
 
 	modem->powered_pending = powered;
 
-	if (modem->powered == powered)
-		return;
+	if (modem->powered != powered) {
+		modem->powered = powered;
 
-	modem->powered = powered;
+		if (modem->driver) {
+			dbus_bool_t dbus_powered = powered;
 
-	if (modem->driver == NULL)
-		return;
-
-	dbus_powered = powered;
-	ofono_dbus_signal_property_changed(conn, modem->path,
+			ofono_dbus_signal_property_changed(conn, modem->path,
 						OFONO_MODEM_INTERFACE,
 						"Powered", DBUS_TYPE_BOOLEAN,
 						&dbus_powered);
+		}
 
-	if (powered) {
-		if (modem->driver->pre_sim)
-			modem->driver->pre_sim(modem);
-	} else {
-		remove_all_atoms(modem);
+		if (powered) {
+			if (modem->driver->pre_sim)
+				modem->driver->pre_sim(modem);
+		}
+	}
+
+	if (powering_down) {
+		modems_remaining -= 1;
+
+		if (modems_remaining == 0)
+			__ofono_exit();
 	}
 }
 
@@ -1086,6 +1113,9 @@ int ofono_modem_register(struct ofono_modem *modem)
 	if (modem == NULL)
 		return -EINVAL;
 
+	if (powering_down == TRUE)
+		return -EBUSY;
+
 	if (modem->driver != NULL)
 		return -EALREADY;
 
@@ -1143,10 +1173,8 @@ static void modem_unregister(struct ofono_modem *modem)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 
-	if (modem->driver == NULL)
-		return;
-
-	remove_all_atoms(modem);
+	if (modem->powered == TRUE)
+		set_powered(modem, FALSE);
 
 	__ofono_watchlist_free(modem->atom_watches);
 	modem->atom_watches = NULL;
@@ -1175,10 +1203,7 @@ static void modem_unregister(struct ofono_modem *modem)
 
 	g_dbus_unregister_interface(conn, modem->path, OFONO_MODEM_INTERFACE);
 
-	if (modem->powered == TRUE)
-		set_powered(modem, FALSE);
-
-	if (modem->driver->remove)
+	if (modem->driver && modem->driver->remove)
 		modem->driver->remove(modem);
 
 	g_hash_table_destroy(modem->properties);
@@ -1222,7 +1247,43 @@ int ofono_modem_driver_register(const struct ofono_modem_driver *d)
 
 void ofono_modem_driver_unregister(const struct ofono_modem_driver *d)
 {
+	GSList *l;
+	struct ofono_modem *modem;
+
 	DBG("driver: %p, name: %s", d, d->name);
 
 	g_driver_list = g_slist_remove(g_driver_list, (void *)d);
+
+	for (l = g_modem_list; l; l = l->next) {
+		modem = l->data;
+
+		if (modem->driver != d)
+			continue;
+
+		modem_unregister(modem);
+	}
+}
+
+void __ofono_modem_shutdown()
+{
+	struct ofono_modem *modem;
+	GSList *l;
+
+	powering_down = TRUE;
+
+	for (l = g_modem_list; l; l = l->next) {
+		modem = l->data;
+
+		if (modem->driver == NULL)
+			continue;
+
+		if (modem->powered == FALSE && modem->powered_pending == FALSE)
+			continue;
+
+		if (set_powered(modem, FALSE) == -EINPROGRESS)
+			modems_remaining += 1;
+	}
+
+	if (modems_remaining == 0)
+		__ofono_exit();
 }
